@@ -142,6 +142,7 @@ static GSList *pagelist = NULL;
 static GHashTable *showing_tabs;
 static char *treepath = NULL;
 static GtkWidget *focus_widget = NULL;
+static int tree_id_num = 0;
 
 
 
@@ -271,7 +272,7 @@ v_align(disp_vert d) {
 }
 
 static void
-widget_dump_tree(widget_info *w) {
+widget_do_dump_tree(widget_info *w) {
 	/* if this widget is dumpable, print its value */
 	switch (w->type) {
 	case C_CHECKBOX:
@@ -304,9 +305,9 @@ widget_dump_tree(widget_info *w) {
 	case C_TREESUB: {
 		tree_item_data *item = w->data;
 		if (item->expand) {
-			static int idnum = 0;
 			char *tpath = parse_get_tree_path((command *)w);
-			io_write_message("dump %s t%d %s\n", w->id, idnum++, tpath);
+			io_write_message("dump %s t%d %s\n",
+					 w->id, tree_id_num++, tpath);
 		}}
 	default:
 		/* not a printable item */
@@ -314,7 +315,13 @@ widget_dump_tree(widget_info *w) {
 	}
 
 	/* recurse */
-	g_slist_foreach (w->child_list, (GFunc) widget_dump_tree, NULL);
+	g_slist_foreach (w->child_list, (GFunc) widget_do_dump_tree, NULL);
+}
+
+static void
+widget_dump_tree(widget_info *w) {
+	tree_id_num = 0;
+	widget_do_dump_tree(w);
 }
 
 static void
@@ -337,6 +344,7 @@ static void
 button_dump_form (GtkWidget *widget, gpointer data) {
 	widget_info *w = get_widget_info(widget);
 
+	if (!w) return;
 	widget_dump_tree((widget_info *)w->form);
 	report_action (widget, data);
 }
@@ -346,7 +354,7 @@ subtree_expand (GtkWidget *widget, gpointer data) {
 	widget_info *w = get_widget_info(widget);
 	tree_item_data *item = w->data;
 
-	item->expand = data;
+	item->expand = (int) data; /* we know data will be 0 or 1 */
 }
 
 static void
